import matplotlib.pyplot as plt

plt.rcParams.update({
    
    'lines.linewidth': 3,
    'lines.color': '#929591',        # Default color for lines
    'axes.labelsize': 20,            # Label font size
    'axes.titlesize': 20,            # Title font size
    'axes.labelcolor': 'black',      # Color of axis labels
    'xtick.labelsize': 15,           # X tick font size
    'ytick.labelsize': 15,           # Y tick font size
    'xtick.direction': 'in',         # X tick direction
    'ytick.direction': 'in',         # Y tick direction
    'legend.fontsize': 22,           # Legend font size
    'legend.frameon': False,         # No frame around legend
    'legend.loc': 'upper center',    # Default legend position
    'figure.figsize': (35, 15)       # Default figure size
})